import { Cloneable, CodeEngine as ICodeEngine, CodeEngineEventEmitter, Logger, PluginDefinition, Summary } from "@code-engine/types";
import { Config } from "./config";
declare const codeEngineEventEmitter: new () => CodeEngineEventEmitter;
/**
 * The main CodeEngine class.
 */
export declare class CodeEngine extends codeEngineEventEmitter implements ICodeEngine {
    constructor(config?: Config);
    /**
     * Returns all CodeEngine instances that have been created and not yet disposed.
     */
    static get instances(): ReadonlyArray<CodeEngine>;
    /**
     * The directory that should be used to resolve all relative paths.
     */
    get cwd(): string;
    /**
     * The number of files that CodeEngine can process concurrently.
     */
    get concurrency(): number;
    /**
     * Indicates whether CodeEngine is running in local development mode.
     * When `true`, plugins should generate files that are un-minified, un-obfuscated, and may
     * contain references to localhost.
     */
    get dev(): boolean;
    /**
     * Indicates whether CodeEngine is running in debug mode, which enables additional logging
     * and error stack traces.
     */
    get debug(): boolean;
    /**
     * Indicates whether the `dispose()` method has been called.
     * Once disposed, the CodeEngine instance is no longer usable.
     */
    get disposed(): boolean;
    /**
     * logs messages and errors
     */
    get log(): Logger;
    /**
     * Loads one or more CodeEngine plugins.
     */
    use(...plugins: PluginDefinition[]): Promise<void>;
    /**
     * Imports one or more JavaScript modules in all worker threads.
     */
    import(moduleId: string, data?: Cloneable): Promise<void>;
    /**
     * Deletes the contents of the destination(s).
     */
    clean(): Promise<void>;
    /**
     * Runs CodeEngine, processing all source files using all currently-loaded plugins.
     */
    run(): Promise<Summary>;
    /**
     * Watches source files for changes and performs incremental runs whenever changes are detected.
     *
     * @param delay
     * The time (in milliseconds) to wait after a file change is detected before starting a run.
     * This allows multiple files that are changed together to all be re-built together.
     */
    watch(delay?: number): void;
    /**
     * Releases system resources that are held by this CodeEngine instance.
     * Once `dispose()` is called, the CodeEngine instance is no longer usable.
     */
    dispose(): Promise<void>;
    /**
     * Disposes all CodeEngine instances.
     */
    static disposeAll(): Promise<void>;
    /**
     * Returns a string representation of the CodeEngine instance.
     */
    toString(): string;
    /**
     * Returns the name to use for `Object.toString()`.
     */
    get [Symbol.toStringTag](): string;
}
export {};
