"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const validate_1 = require("@code-engine/validate");
const workers_1 = require("@code-engine/workers");
const ono_1 = require("@jsdevtools/ono");
const events_1 = require("events");
const os = require("os");
const mount_1 = require("./plugins/mount");
const plugin_controller_1 = require("./plugins/plugin-controller");
const pipeline_1 = require("./run/pipeline");
const instances = [];
// Hacky workaround to inherit from EventEmitter while also implementing the CodeEngineEventEmitter interface
const codeEngineEventEmitter = events_1.EventEmitter;
/**
 * The main CodeEngine class.
 */
class CodeEngine extends codeEngineEventEmitter {
    constructor(config = {}) {
        super();
        /** @internal */
        this._disposed = false;
        this._cwd = config.cwd || process.cwd();
        this._concurrency = validate_1.validate.number.integer.positive(config.concurrency, "concurrency", os.cpus().length);
        this._dev = config.dev === undefined ? process.env.NODE_ENV === "development" : config.dev;
        this._debug = config.debug === undefined ? Boolean(process.env.DEBUG) : config.debug;
        this._log = utils_1.createLogEmitter(this, this.debug);
        this._pipeline = new pipeline_1.Pipeline(this);
        this._workerPool = new workers_1.WorkerPool(this);
        instances.push(this);
    }
    /**
     * Returns all CodeEngine instances that have been created and not yet disposed.
     */
    static get instances() {
        return instances.slice();
    }
    /**
     * The directory that should be used to resolve all relative paths.
     */
    get cwd() {
        return this._cwd;
    }
    /**
     * The number of files that CodeEngine can process concurrently.
     */
    get concurrency() {
        return this._concurrency;
    }
    /**
     * Indicates whether CodeEngine is running in local development mode.
     * When `true`, plugins should generate files that are un-minified, un-obfuscated, and may
     * contain references to localhost.
     */
    get dev() {
        return this._dev;
    }
    /**
     * Indicates whether CodeEngine is running in debug mode, which enables additional logging
     * and error stack traces.
     */
    get debug() {
        return this._debug;
    }
    /**
     * Indicates whether the `dispose()` method has been called.
     * Once disposed, the CodeEngine instance is no longer usable.
     */
    get disposed() {
        return this._disposed;
    }
    /**
     * logs messages and errors
     */
    get log() {
        return this._log;
    }
    /**
     * Loads one or more CodeEngine plugins.
     */
    async use(...plugins) {
        this._assertNotDisposed();
        for (let pluginDefinition of plugins) {
            // Normalize the plugin
            let defaultName = `Plugin ${this._pipeline.size + 1}`;
            let plugin = await mount_1.mountPlugin(this, this._workerPool, pluginDefinition, defaultName);
            let controller = new plugin_controller_1.PluginController(plugin);
            // Add the plugin to the pipeline
            this._pipeline.add(controller);
            // Initialize the plugin, if necessary
            if (controller.initialize) {
                await controller.initialize();
            }
        }
    }
    /**
     * Imports one or more JavaScript modules in all worker threads.
     */
    async import(moduleId, data) {
        this._assertNotDisposed();
        await this._workerPool.importModule(moduleId, data);
    }
    /**
     * Deletes the contents of the destination(s).
     */
    async clean() {
        this._assertNotDisposed();
        return this._pipeline.clean();
    }
    /**
     * Runs CodeEngine, processing all source files using all currently-loaded plugins.
     */
    async run() {
        this._assertNotDisposed();
        try {
            let summary = await this._pipeline.run();
            return summary;
        }
        catch (error) {
            // Emit the "Error" event if there are any handlers registered
            if (this.listenerCount("error" /* Error */) > 0) {
                this.emit("error" /* Error */, error);
            }
            // Re-throw the error, regardless of whether there were event handlers
            throw error;
        }
    }
    /**
     * Watches source files for changes and performs incremental runs whenever changes are detected.
     *
     * @param delay
     * The time (in milliseconds) to wait after a file change is detected before starting a run.
     * This allows multiple files that are changed together to all be re-built together.
     */
    watch(delay) {
        delay = validate_1.validate.number.integer.positive(delay, "watch delay", 300);
        this._assertNotDisposed();
        this._pipeline.watch(delay);
    }
    /**
     * Releases system resources that are held by this CodeEngine instance.
     * Once `dispose()` is called, the CodeEngine instance is no longer usable.
     */
    async dispose() {
        if (this._disposed) {
            return;
        }
        this._disposed = true;
        let index = instances.indexOf(this);
        index >= 0 && instances.splice(index, 1);
        await Promise.all([
            this._pipeline.dispose(),
            this._workerPool.dispose(),
        ]);
    }
    /**
     * Disposes all CodeEngine instances.
     */
    // tslint:disable-next-line: member-ordering
    static async disposeAll() {
        await Promise.all(instances.splice(0, instances.length).map((engine) => engine.dispose()));
    }
    /**
     * Returns a string representation of the CodeEngine instance.
     */
    toString() {
        if (this._disposed) {
            return `CodeEngine (disposed)`;
        }
        else {
            return `CodeEngine (${this._pipeline.size} plugins)`;
        }
    }
    /**
     * Returns the name to use for `Object.toString()`.
     */
    get [Symbol.toStringTag]() {
        return "CodeEngine";
    }
    /**
     * Throws an error if the `CodeEngine` has been disposed.
     * @internal
     */
    _assertNotDisposed() {
        if (this.disposed) {
            throw ono_1.ono(`CodeEngine cannot be used after it has been disposed.`);
        }
    }
}
exports.CodeEngine = CodeEngine;
//# sourceMappingURL=code-engine.js.map