"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stringify_1 = require("@code-engine/stringify");
const ono_1 = require("@jsdevtools/ono");
const types_1 = require("./types");
/**
 * Mounts any `PluginDefinition` value, returning a sanitized and normalized plugin.
 * @internal
 */
async function mountPlugin(engine, workerPool, definition, defaultName) {
    let name;
    let processFile;
    try {
        let normalized = normalizePluginDefinition(definition);
        name = normalized.name;
        if (types_1.isModuleDefinition(normalized.processFile)) {
            // Import the FileProcessor on all worker threads
            let { moduleId, data } = normalized.processFile;
            processFile = await workerPool.importFileProcessor(moduleId, data);
            // Use the FileProcessor's name as the plugin name, if necessary
            name = name || processFile.name;
        }
        else {
            processFile = normalized.processFile;
        }
        // Fallback to a default name if no name was specified
        name = String(name || defaultName);
        // Replace/set the properties of the original plugin object
        return Object.assign(normalized, { engine, name, processFile });
    }
    catch (error) {
        throw ono_1.ono(error, `Error in ${name || defaultName}.`);
    }
}
exports.mountPlugin = mountPlugin;
/**
 * Normalizes a `PluginDefinition` value to a `Plugin` object.
 * @internal
 */
function normalizePluginDefinition(definition) {
    let normalized;
    if (typeof definition === "function") {
        // This plugin is just a main-thread processFile() method
        normalized = { name: definition.name, processFile: definition };
    }
    else if (typeof definition === "string") {
        // This plugin is just a worker-thread processFile() method
        normalized = { processFile: { moduleId: definition } };
    }
    else if (types_1.isModuleDefinition(definition)) {
        // This plugin is just a worker-thread processFile() method
        normalized = { processFile: definition };
    }
    else if (types_1.isPlugin(definition)) {
        normalized = definition;
        if (typeof definition.processFile === "string") {
            normalized.processFile = { moduleId: definition.processFile };
        }
    }
    else {
        throw ono_1.ono.type(`Invalid CodeEngine plugin: ${stringify_1.stringify(definition)}`);
    }
    return normalized;
}
exports.normalizePluginDefinition = normalizePluginDefinition;
//# sourceMappingURL=mount.js.map