"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const file_path_filter_1 = require("@jsdevtools/file-path-filter");
const ono_1 = require("@jsdevtools/ono");
/**
 * Wraps a plugin's functionality in a consistent interface.
 * @internal
 */
class PluginController {
    constructor(plugin) {
        this.name = plugin.name;
        this.filter = file_path_filter_1.createFilter({ map }, plugin.filter === undefined ? true : plugin.filter);
        this._plugin = plugin;
        plugin.initialize || (this.initialize = undefined);
        plugin.processFile || (this.processFile = undefined);
        plugin.processFiles || (this.processFiles = undefined);
        plugin.read || (this.read = undefined);
        plugin.watch || (this.watch = undefined);
        plugin.clean || (this.clean = undefined);
        plugin.dispose || (this.dispose = undefined);
    }
    async initialize() {
        try {
            await this._plugin.initialize();
        }
        catch (error) {
            throw ono_1.ono(error, `An error occurred in ${this} while it was initializing.`);
        }
    }
    // tslint:disable-next-line: no-async-without-await
    async *processFiles(files, run) {
        try {
            let fileInfos = this._plugin.processFiles(files, run);
            for await (let fileInfo of utils_1.iterate(fileInfos)) {
                yield utils_1.createFile(fileInfo, this.name);
            }
            // Ensure that all input files are read, even if the plugin doesn't actually read them.
            // Otherwise the run will never complete.
            await utils_1.drainIterable(files);
        }
        catch (error) {
            throw ono_1.ono(error, `An error occurred in ${this} while processing files.`);
        }
    }
    async processFile(file, run, output) {
        try {
            let fileInfos = this._plugin.processFile(file, run);
            for await (let fileInfo of utils_1.iterate(fileInfos)) {
                await output.write(utils_1.createFile(fileInfo, this.name));
            }
        }
        catch (error) {
            throw ono_1.ono(error, `An error occurred in ${this} while processing ${file}.`);
        }
    }
    read(run) {
        let iterator;
        try {
            // NOTE: The reason we don't just use a `for await...of` loop here is that we want to be able to
            // ead multiple files simultaneously if the `processFile()` plugins are faster than the IO source.
            let fileInfos = this._plugin.read(run);
            iterator = utils_1.iterate(fileInfos)[Symbol.asyncIterator]();
        }
        catch (error) {
            throw ono_1.ono(error, `An error occurred in ${this} while reading source files.`);
        }
        let next = async () => {
            try {
                let result = await iterator.next();
                if (result.done) {
                    return result;
                }
                let file = utils_1.createFile(result.value, this.name);
                return { value: file };
            }
            catch (error) {
                throw ono_1.ono(error, `An error occurred in ${this} while reading source files.`);
            }
        };
        return { [Symbol.asyncIterator]: () => ({ next }) };
    }
    // tslint:disable-next-line: no-async-without-await
    async *watch() {
        try {
            // Create a callback function that yields a changed file
            let writer = new utils_1.IterableWriter();
            let callback = (file) => writer.write(file);
            let changedFileInfos = this._plugin.watch(callback);
            if (changedFileInfos) {
                // The plugin returned an async iterable, so read from it
                // in addition to any values that are written via the callback function.
                let iterator = utils_1.iterate(changedFileInfos)[Symbol.asyncIterator]();
                writer.onRead = async () => {
                    try {
                        let result = await iterator.next();
                        if (result.done) {
                            await writer.end();
                        }
                        else {
                            await writer.write(result.value);
                        }
                    }
                    catch (error) {
                        await writer.throw(error);
                    }
                };
            }
            for await (let changedFileInfo of writer.iterable) {
                let file = utils_1.createChangedFile(changedFileInfo, this.name);
                // Return an additional flag that indicates whether any contents were actually specified.
                // This is necessary because the ChangedFile object has defaults applied.
                let hasContents = Boolean(changedFileInfo.contents || changedFileInfo.text);
                yield { file, hasContents };
            }
        }
        catch (error) {
            throw ono_1.ono(error, `An error occurred in ${this} while watching source files for changes.`);
        }
    }
    async clean() {
        try {
            await this._plugin.clean();
        }
        catch (error) {
            throw ono_1.ono(error, `An error occurred in ${this} while cleaning the destination.`);
        }
    }
    async dispose() {
        try {
            await this._plugin.dispose();
        }
        catch (error) {
            throw ono_1.ono(error, `An error occurred in ${this} while cleaning-up.`);
        }
    }
    /**
     * Returns a string representation of the plugin.
     */
    toString() {
        return this.name;
    }
}
exports.PluginController = PluginController;
/**
 * Maps `File` objects to paths for filtering
 */
function map(file) {
    return file.path;
}
//# sourceMappingURL=plugin-controller.js.map