"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const types_1 = require("../plugins/types");
const run_pipeline_1 = require("./run-pipeline");
const watch_1 = require("./watch");
/**
 * A sequence of CodeEngine plugins that process files.
 * @internal
 */
class Pipeline {
    constructor(engine) {
        this._plugins = [];
        this._engine = engine;
    }
    /**
     * The number of plugins in the pipeline.
     */
    get size() {
        return this._plugins.length;
    }
    /**
     * Adds a plugin to the pipeline
     */
    add(plugin) {
        this._plugins.push(plugin);
    }
    /**
     * Deletes the contents of the destination(s).
     */
    async clean() {
        let cleaners = this._plugins.filter(types_1.isCleanerPlugin);
        await Promise.all(cleaners.map((plugin) => plugin.clean()));
    }
    /**
     * Runs all source files through the plugin pipeline.
     */
    async run() {
        let run = createRun(this._engine, {
            full: true,
            partial: false,
            changedFiles: [],
        });
        this._emitStart(run);
        let files = readAllSources(this._plugins, run);
        let steps = this._plugins.filter(types_1.isFileProcessorPlugin);
        let summary = await run_pipeline_1.runPipeline(files, steps, run);
        this._emitFinish(summary);
        return summary;
    }
    /**
     * Watches source files for changes and starts an incremental run whenever changes are detected.
     */
    watch(delay) {
        let steps = this._plugins.filter(types_1.isFileProcessorPlugin);
        Promise.resolve()
            .then(async () => {
            for await (let changedFiles of watch_1.watchAllSources(this._plugins, this._engine, delay)) {
                let run = createRun(this._engine, {
                    full: false,
                    partial: true,
                    changedFiles,
                });
                this._emitStart(run);
                // Only run the files that still exist (i.e. not the ones that were deleted)
                let files = changedFiles.filter((file) => file.change !== "deleted" /* Deleted */);
                let summary = await run_pipeline_1.runPipeline(utils_1.iterate(files), steps, run);
                this._emitFinish(summary);
            }
        })
            .catch((error) => {
            this._engine.emit("error" /* Error */, error);
        });
    }
    /**
     * Removes all plugins and stops watching source files.
     */
    async dispose() {
        let plugins = this._plugins;
        this._plugins = [];
        let needDisposed = plugins.filter(types_1.isDisposablePlugin);
        await Promise.all(needDisposed.map((plugin) => plugin.dispose()));
    }
    /**
     * Emits a "start" event
     */
    _emitStart(run) {
        if (this._engine.listenerCount("start" /* Start */) > 0) {
            // Clone the Run to prevent listeners from mutating it and affecting the run
            run = {
                ...run,
                changedFiles: [
                    ...run.changedFiles
                ]
            };
            this._engine.emit("start" /* Start */, run);
        }
    }
    /**
     * Emits a "finish" event
     */
    _emitFinish(summary) {
        if (this._engine.listenerCount("finish" /* Finish */) > 0) {
            this._engine.emit("finish" /* Finish */, summary);
        }
    }
}
exports.Pipeline = Pipeline;
/**
 * Creates a new `Run` object with the specified properties.
 */
function createRun(engine, props) {
    return {
        cwd: engine.cwd,
        concurrency: engine.concurrency,
        full: true,
        partial: false,
        debug: engine.debug,
        dev: engine.dev,
        changedFiles: [],
        log: engine.log,
        ...props,
    };
}
/**
 * Reads all source files simultaneously from all file sources.
 */
function readAllSources(plugins, run) {
    let sources = plugins.filter(types_1.isFileSourcePlugin);
    let fileGenerators = sources.map((source) => source.read(run));
    return utils_1.joinIterables(...fileGenerators);
}
//# sourceMappingURL=pipeline.js.map