"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const run_step_1 = require("./run-step");
/**
 * Runs files through a series of plugins.
 * @internal
 */
async function runPipeline(files, steps, run) {
    let promises = [], promise;
    // Remove the contents from the changed files so the Run is lightweight
    // for cloning across the thread boundary
    // @ts-ignore
    run.changedFiles = run.changedFiles.map(lightweightChangedFile);
    let summary = {
        ...run,
        input: { fileCount: 0, fileSize: 0 },
        output: { fileCount: 0, fileSize: 0 },
        time: { start: new Date(), end: new Date(), elapsed: 0 },
    };
    // Collect metrics on the input files
    let input = updateSummary(summary, "input", files);
    // Chain the steps together, with each one accepting the output of the previous one as input
    for (let step of steps) {
        let output = new utils_1.IterableWriter();
        promise = run_step_1.runStep(step, input, output, run);
        promises.push(promise);
        input = output.iterable;
    }
    // Collect metrics on the final output files
    let finalOutput = updateSummary(summary, "output", input);
    // Wait for all steps to finish
    promises.push(utils_1.drainIterable(finalOutput, run.concurrency));
    await Promise.all(promises);
    // Update the summary
    summary.time.end = new Date();
    summary.time.elapsed = summary.time.end.getTime() - summary.time.start.getTime();
    return summary;
}
exports.runPipeline = runPipeline;
/**
 * Creates a "lightweight" copy of a `ChangedFile` object without its contents.
 */
function lightweightChangedFile(changedFile) {
    let copy = utils_1.createChangedFile({ ...changedFile });
    copy.contents = Buffer.alloc(0);
    return copy;
}
/**
 * Updates the `input` or `output` metrics of the given `Summary`.
 */
function updateSummary(summary, io, files) {
    return {
        [Symbol.asyncIterator]() {
            let iterator = files[Symbol.asyncIterator]();
            return {
                async next() {
                    let result = await iterator.next();
                    if (!result.done) {
                        summary[io].fileCount++;
                        summary[io].fileSize += result.value.size;
                    }
                    return result;
                }
            };
        },
    };
}
//# sourceMappingURL=run-pipeline.js.map